--
--	WhenLevel
--	Written by scott@mithrandir.com
--
--	Version History
--		001	Initial version that records when leveling is done.
--		002	Added command line to output leveling information.
--		003	Converted to using strings in preparation for localization.
--		004	Computes maximum level instead of hardcoding 80.
--
--	UTF-8 file
--

WhenLevel_File_Version = 004

if nil == WhenLevel or WhenLevel.versionNumber < WhenLevel_File_Version then

	--	This is the data that is written to disk that records each level date and time played.
	WhenLevelData = { }

	--	The actual addon itself.
	--	Conceptually a very simple addon that creates a tooltip that is used to receive events for the addon.
	--	The tooltip is hidden and is not used for any other purpose.  A slash command is registered so the
	--	user can request the report of leveling information.  Two events are of importance -- PLAYER_LEVEL_UP
	--	and TIME_PLAYED_MSG.  When the addon receives notification that PLAYER_LEVEL_UP it ensures the server
	--	is asked for how much time is played, causing TIME_PLAYED_MSG to eventually be sent.  Note that within
	--	the processing of the PLAYER_LEVEL_UP event the call to UnitLevel() will not return the new level, so
	--	we use the first argument in the PLAYER_LEVEL_UP event to record this information.  Also note that upon
	--	entry into the world the server now reports the time played so we also receive TIME_PLAYED_MSG.  In
	--	processing TIME_PLAYED_MSG we record the server time and the total time played if we have not already
	--	recorded this information for the current character level.  Since we only record the time played at a
	--	level once, it is safe for the user to use the /played slash command without harming the WhenLevel data.
	--	Since a lot of characters will first use WhenLevel after level one, it is a special case and noted in
	--	the /whenlevel output data.
	WhenLevel = {

		versionNumber = WhenLevel_File_Version,
		currentlyProcessingLevel,
		eventDispatch = {			-- table of functions whose keys are the events
			['PLAYER_ENTERING_WORLD'] = function(self)
				self.locale = GetLocale()
				-- if the locale is not found to be supported default to enUS
				if nil == self.s[self.locale] then
					self.locale = 'enUS'
				end
				self.currentlyProcessingLevel = UnitLevel("player")
				this:RegisterEvent("PLAYER_LEVEL_UP")
				this:RegisterEvent("TIME_PLAYED_MSG")
			end,
			['PLAYER_LEVEL_UP'] = function(self, arg1)
				--	arg1 tells us the new level, but we just cause another event to be posted
				self.currentlyProcessingLevel = arg1
				RequestTimePlayed()
			end,
			['TIME_PLAYED_MSG'] = function(self, arg1, arg2)
				--	arg1 tells us total time
				--	arg2 tells us total time at level
				local currentLevel = self.currentlyProcessingLevel
				if (nil == WhenLevelData[currentLevel]) then
					local hour, minute = GetGameTime()
					local weekday, month, day, year = CalendarGetDate()
					WhenLevelData[currentLevel] = { }
					WhenLevelData[currentLevel]["totalPlayed"] = arg1
					WhenLevelData[currentLevel]["serverTime"] = string.format("%4d-%02d-%02d %02d:%02d", year, month, day, hour, minute)
				end
			end,
			['VARIABLES_LOADED'] = function(self)
				--	Ensure the tooltip is not messed up
				if not com_mithrandir_whenLevelTooltip:IsOwned(UIParent) then
					com_mithrandir_whenLevelTooltip:SetOwner(UIParent, "ANCHOR_NONE")
				end
				this:RegisterEvent("PLAYER_ENTERING_WORLD")
			end,
			},
		locale,
		--	This section has the localization information for all the output text that the addon generates.
		--	Note that it is done this way solely to minimize the namespace pollution this addon causes.
		--	The downfall of this method is a larger runtime size since each of the localizations is kept
		--	in memory, and the need for detecting whether the locale the user is using is actually supported.
		--	We could eliminate the runtime size issue by eliminating the non-used locales, but that would
		--	require a little more code and the desire is smaller.  The number of strings used in this addon
		--	do not require that much memory, so the desire for less namespace pollution wins.
		s = {
			['enUS'] = {
				FIRST_USED_FORMAT   = 'WhenLevel first used at Level %d',
				LEVEL_GAINED_FORMAT = 'Level %d on %s',
				PLAYED_FORMAT       = ' played %s [%s]',
				DAYS                = 'd',
				HOURS               = 'h',
				MINUTES             = 'm',
				SECONDS             = 's',
				},
			['deDE'] = {
				FIRST_USED_FORMAT   = 'WhenLevel first used at Level %d',
				LEVEL_GAINED_FORMAT = 'Level %d on %s',
				PLAYED_FORMAT       = ' played %s [%s]',
				DAYS                = 'T',
				HOURS               = 'H',
				MINUTES             = 'M',
				SECONDS             = 'S',
				},
			['enGB'] = {
				FIRST_USED_FORMAT   = 'WhenLevel first used at Level %d',
				LEVEL_GAINED_FORMAT = 'Level %d on %s',
				PLAYED_FORMAT       = ' played %s [%s]',
				DAYS                = 'd',
				HOURS               = 'h',
				MINUTES             = 'm',
				SECONDS             = 's',
				},
			['esES'] = {
				FIRST_USED_FORMAT   = 'WhenLevel first used at Level %d',
				LEVEL_GAINED_FORMAT = 'Level %d on %s',
				PLAYED_FORMAT       = ' played %s [%s]',
				DAYS                = 'd',
				HOURS               = 'h',
				MINUTES             = 'm',
				SECONDS             = 's',
				},
			['esMX'] = {
				FIRST_USED_FORMAT   = 'WhenLevel first used at Level %d',
				LEVEL_GAINED_FORMAT = 'Level %d on %s',
				PLAYED_FORMAT       = ' played %s [%s]',
				DAYS                = 'd',
				HOURS               = 'h',
				MINUTES             = 'm',
				SECONDS             = 's',
				},
			['frFR'] = {
				FIRST_USED_FORMAT   = 'WhenLevel first used at Level %d',
				LEVEL_GAINED_FORMAT = 'Level %d on %s',
				PLAYED_FORMAT       = ' played %s [%s]',
				DAYS                = 'j',
				HOURS               = 'h',
				MINUTES             = 'm',
				SECONDS             = 's',
				},
			['koKR'] = {
				FIRST_USED_FORMAT   = 'WhenLevel first used at Level %d',
				LEVEL_GAINED_FORMAT = 'Level %d on %s',
				PLAYED_FORMAT       = ' played %s [%s]',
				DAYS                = 'd',
				HOURS               = 'h',
				MINUTES             = 'm',
				SECONDS             = 's',
				},
			['ruRU'] = {
				FIRST_USED_FORMAT   = 'WhenLevel first used at Level %d',
				LEVEL_GAINED_FORMAT = 'Level %d on %s',
				PLAYED_FORMAT       = ' played %s [%s]',
				DAYS                = 'd',
				HOURS               = 'h',
				MINUTES             = 'm',
				SECONDS             = 's',
				},
			['zhCN'] = {
				FIRST_USED_FORMAT   = 'WhenLevel first used at Level %d',
				LEVEL_GAINED_FORMAT = 'Level %d on %s',
				PLAYED_FORMAT       = ' played %s [%s]',
				DAYS                = 'd',
				HOURS               = 'h',
				MINUTES             = 'm',
				SECONDS             = 's',
				},
			['zhTW'] = {
				FIRST_USED_FORMAT   = 'WhenLevel first used at Level %d',
				LEVEL_GAINED_FORMAT = 'Level %d on %s',
				PLAYED_FORMAT       = ' played %s [%s]',
				DAYS                = 'd',
				HOURS               = 'h',
				MINUTES             = 'm',
				SECONDS             = 's',
				},
			},

		FormattedSeconds = function(self, totalSeconds)
			local days = math.floor(totalSeconds / 86400)
			local remainder = totalSeconds - (days * 86400)
			local hours = math.floor(remainder / 3600)
			remainder = remainder - (hours * 3600)
			local minutes = math.floor(remainder / 60)
			local seconds = remainder - (minutes * 60)
			local retval = ""

			if (days > 0) then retval = retval .. days .. self.s[self.locale]['DAYS'] end
			if (hours > 0 or days > 0) then retval = retval .. hours .. self.s[self.locale]['HOURS'] end
			if (minutes > 0 or hours > 0 or days > 0) then retval = retval .. minutes .. self.s[self.locale]['MINUTES'] end
			retval = retval .. seconds .. self.s[self.locale]['SECONDS']
			return retval
		end,

		OutputLevelMessage = function(self, level)
			local outputMessage = string.format(self.s[self.locale]['LEVEL_GAINED_FORMAT'], level, WhenLevelData[level]["serverTime"])
			if (nil ~= WhenLevelData[level + 1]) then
				local totalSeconds = WhenLevelData[level + 1]["totalPlayed"] - WhenLevelData[level]["totalPlayed"]
				outputMessage = outputMessage .. string.format(self.s[self.locale]['PLAYED_FORMAT'], self:FormattedSeconds(totalSeconds), self:FormattedSeconds(WhenLevelData[level + 1]["totalPlayed"]))
			end
			DEFAULT_CHAT_FRAME:AddMessage(outputMessage)
		end,

		SlashCommand = function(self, msg)
			local firstLevelEncountered = nil
			local whereToStop = UnitLevel('player')
			for i = 1, whereToStop, 1 do
				if (nil ~= WhenLevelData[i]) then
					if (nil == firstLevelEncountered) then
						firstLevelEncountered = i
						if (1 == i) then
							self:OutputLevelMessage(i)
						else
							DEFAULT_CHAT_FRAME:AddMessage(string.format(self.s[self.locale]['FIRST_USED_FORMAT'], firstLevelEncountered))
						end
					else
						self:OutputLevelMessage(i)
					end
					
				end
			end
		end,

		Tooltip_OnEvent = function(self)
			if self.eventDispatch[event] then
				self.eventDispatch[event](self, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
			end
		end,

		Tooltip_OnLoad = function(self)
			SlashCmdList["WHENLEVEL"] = function(msg)
				self:SlashCommand(msg)
			end
			SLASH_WHENLEVEL1 = "/whenlevel"
			this:SetScript("OnEvent", function() self:Tooltip_OnEvent() end)
			this:RegisterEvent("VARIABLES_LOADED")
		end,

		}

end
